@@ -65,13 +65,7 @@ static const char *rcsid = "tftp-hpa $Id: tftp.c,v 1.1.1.1 1999/09/26 06:32:42 h
 #include "extern.h"
 #include "tftpsubs.h"
 
-void bsd_signal(int, void (*)(int));
-
-#ifndef HAVE_SIGSETJMP
-#define sigsetjmp(x,y)  setjmp(x)
-#define siglongjmp(x,y) longjmp(x,y)
-#define sigjmp_buf jmp_buf
-#endif
+extern	int errno;
 
 extern  struct sockaddr_in peeraddr;	/* filled in by main */
 extern  int     f;			/* the opened socket */
@@ -90,8 +84,8 @@ extern  int     maxtimeout;
 #define PKTSIZE    SEGSIZE+4
 char    ackbuf[PKTSIZE];
 int	timeout;
-sigjmp_buf	toplevel;
-sigjmp_buf	timeoutbuf;
+jmp_buf	toplevel;
+jmp_buf	timeoutbuf;
 
 static void nak(int);
 static int makerequest(int, const char *, struct tftphdr *, const char *);
@@ -124,7 +118,7 @@ sendfile(int fd, char *name, char *mode)
 	block = 0;
 	amount = 0;
 
-	bsd_signal(SIGALRM, timer);
+	signal(SIGALRM, timer);
 	do {
 		if (block == 0)
 			size = makerequest(WRQ, name, dp, mode) - 4;
@@ -139,7 +133,7 @@ sendfile(int fd, char *name, char *mode)
 			dp->th_block = htons((u_short)block);
 		}
 		timeout = 0;
-		(void) sigsetjmp(timeoutbuf,1);
+		(void) setjmp(timeoutbuf);
 send_data:
 		if (trace)
 			tpacket("sent", dp, size + 4);
@@ -228,7 +222,7 @@ recvfile(int fd, char *name, char *mode)
 	firsttrip = 1;
 	amount = 0;
 
-	bsd_signal(SIGALRM, timer);
+	signal(SIGALRM, timer);
 	do {
 		if (firsttrip) {
 			size = makerequest(RRQ, name, ap, mode);
@@ -240,7 +234,7 @@ recvfile(int fd, char *name, char *mode)
 			block++;
 		}
 		timeout = 0;
-		(void) sigsetjmp(timeoutbuf,1);
+		(void) setjmp(timeoutbuf);
 send_ack:
 		if (trace)
 			tpacket("sent", ap, size);
@@ -452,8 +446,8 @@ timer(int sig)
 	if (timeout >= maxtimeout) {
 		printf("Transfer timed out.\n");
 		errno = save_errno;
-		siglongjmp(toplevel, -1);
+		longjmp(toplevel, -1);
 	}
 	errno = save_errno;
-	siglongjmp(timeoutbuf, 1);
+	longjmp(timeoutbuf, 1);
 }
