@@ -1,4 +1,4 @@
-/* tftp-hpa: $Id: tftpd.c,v 1.4 1999/09/26 07:12:54 hpa Exp $ */
+/* tftp-hpa: $Id: tftpd.c,v 1.5 2001/03/28 18:24:25 hpa Exp $ */
 
 /* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
@@ -41,7 +41,7 @@ static const char *copyright =
  All rights reserved.\n";
 /*static char sccsid[] = "from: @(#)tftpd.c	5.13 (Berkeley) 2/26/91";*/
 /*static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";*/
-static const char *rcsid = "tftp-hpa $Id: tftpd.c,v 1.4 1999/09/26 07:12:54 hpa Exp $";
+static const char *rcsid = "tftp-hpa $Id: tftpd.c,v 1.5 2001/03/28 18:24:25 hpa Exp $";
 #endif /* not lint */
 
 /*
@@ -73,14 +73,7 @@ static const char *rcsid = "tftp-hpa $Id: tftpd.c,v 1.4 1999/09/26 07:12:54 hpa
 #include <limits.h>
 
 #include "tftpsubs.h"
-
-void bsd_signal(int, void (*)(int));
-
-#ifndef HAVE_SIGSETJMP
-#define sigsetjmp(x,y)  setjmp(x)
-#define siglongjmp(x,y) longjmp(x,y)
-#define sigjmp_buf jmp_buf
-#endif
+#include "recvfrom.h"
 
 #define	TIMEOUT 5		/* Default timeout (seconds) */
 #define TRIES   4		/* Number of attempts to send each packet */
@@ -93,8 +86,8 @@ void bsd_signal(int, void (*)(int));
 #define EOPTNEG	8
 #endif
 
+extern	int errno;
 extern	char *__progname;
-struct	sockaddr_in s_in;
 int	peer;
 int	timeout    = TIMEOUT;
 int	rexmtval   = TIMEOUT;
@@ -154,6 +147,7 @@ main(int argc, char **argv)
 	struct tftphdr *tp;
 	struct passwd *pw;
 	struct options *opt;
+	struct sockaddr_in myaddr;
 	int n = 0;
 	int on = 1;
 	int fd = 0;
@@ -239,8 +233,9 @@ main(int argc, char **argv)
 		exit(1);
 	}
 	fromlen = sizeof (from);
-	n = recvfrom(fd, buf, sizeof (buf), 0,
-	    (struct sockaddr *)&from, &fromlen);
+	n = myrecvfrom(fd, buf, sizeof (buf), 0,
+		       (struct sockaddr *)&from, &fromlen,
+		       &myaddr);
 	if (n < 0) {
 		syslog(LOG_ERR, "recvfrom: %m");
 		exit(1);
@@ -274,8 +269,8 @@ main(int argc, char **argv)
 			 * a single request from a single client.
 			 */
 			j = sizeof from;
-			i = recvfrom(fd, buf, sizeof (buf), 0,
-			    (struct sockaddr *)&from, &j);
+			i = myrecvfrom(fd, buf, sizeof (buf), 0,
+			    (struct sockaddr *)&from, &j, &myaddr);
 			if (i > 0) {
 				n = i;
 				fromlen = j;
@@ -298,9 +293,8 @@ main(int argc, char **argv)
 		syslog(LOG_ERR, "socket: %m");
 		exit(1);
 	}
-	memset(&s_in, 0, sizeof s_in);
-	s_in.sin_family = AF_INET;
-	if (bind(peer, (struct sockaddr *)&s_in, sizeof (s_in)) < 0) {
+	myaddr.sin_port = htons(0); /* We want a new local port */
+	if (bind(peer, (struct sockaddr *)&myaddr, sizeof (myaddr)) < 0) {
 		syslog(LOG_ERR, "bind: %m");
 		exit(1);
 	}
@@ -631,7 +625,7 @@ validate_access(char *filename, int mode, struct formats *pf)
 }
 
 int	timeout;
-sigjmp_buf	timeoutbuf;
+jmp_buf	timeoutbuf;
 
 void
 timer(int sig)
@@ -640,7 +634,7 @@ timer(int sig)
 	timeout += rexmtval;
 	if (timeout >= maxtimeout)
 		exit(0);
-	siglongjmp(timeoutbuf, 1);
+	longjmp(timeoutbuf, 1);
 }
 
 /*
@@ -653,18 +647,18 @@ sendfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 	struct tftphdr *ap;    /* ack packet */
 	int block = 1, size, n;
 
+	signal(SIGALRM, timer);
 	ap = (struct tftphdr *)ackbuf;
 
         if (oap) {
 	     timeout = 0;
-	     (void)sigsetjmp(timeoutbuf,1);
+	     (void)setjmp(timeoutbuf);
 oack:
 	     if (send(peer, oap, oacklen, 0) != oacklen) {
 		  syslog(LOG_ERR, "tftpd: oack: %m\n");
 		  goto abort;
 	     }
 	     for ( ; ; ) {
-	          bsd_signal(SIGALRM, timer);
 		  alarm(rexmtval);
 		  n = recv(peer, ackbuf, sizeof(ackbuf), 0);
 		  alarm(0);
@@ -700,7 +694,7 @@ oack:
 		dp->th_opcode = htons((u_short)DATA);
 		dp->th_block = htons((u_short)block);
 		timeout = 0;
-		(void) sigsetjmp(timeoutbuf,1);
+		(void) setjmp(timeoutbuf);
 
 send_data:
 		if (send(peer, dp, size + 4, 0) != size + 4) {
@@ -709,7 +703,6 @@ send_data:
 		}
 		read_ahead(file, pf->f_convert);
 		for ( ; ; ) {
-		        bsd_signal(SIGALRM, timer);
 			alarm(rexmtval);	/* read the ack */
 			n = recv(peer, ackbuf, sizeof (ackbuf), 0);
 			alarm(0);
@@ -758,6 +751,7 @@ recvfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 	struct tftphdr *ap;    /* ack buffer */
 	int block = 0, acksize, n, size;
 
+	signal(SIGALRM, timer);
 	dp = w_init();
 	do {
 		timeout = 0;
@@ -771,8 +765,7 @@ recvfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 		     ap->th_block = htons((u_short)block);
 		     acksize = 4;
 		}
-		(void) sigsetjmp(timeoutbuf,1);
-		bsd_signal(SIGALRM, timer);
+		(void) setjmp(timeoutbuf);
 send_ack:
 		if (send(peer, ackbuf, acksize, 0) != acksize) {
 			syslog(LOG_ERR, "tftpd: write(ack): %m");
@@ -780,7 +773,6 @@ send_ack:
 		}
 		write_behind(file, pf->f_convert);
 		for ( ; ; ) {
-		        bsd_signal(SIGALRM, timer);
 			alarm(rexmtval);
 			n = recv(peer, dp, PKTSIZE, 0);
 			alarm(0);
@@ -817,7 +809,7 @@ send_ack:
 	ap->th_block = htons((u_short)(block));
 	(void) send(peer, ackbuf, 4, 0);
 
-	bsd_signal(SIGALRM, justquit);      /* just quit on timeout */
+	signal(SIGALRM, justquit);      /* just quit on timeout */
 	alarm(rexmtval);
 	n = recv(peer, buf, sizeof (buf), 0); /* normally times out and quits */
 	alarm(0);
